@@ -1,6 +1,6 @@
 package Bio::Graphics::Glyph::image;
 
-# $Id: image.pm,v 1.2 2009/03/23 17:24:14 lstein Exp $
+# $Id: image.pm,v 1.3 2009/04/02 22:22:07 lstein Exp $
 
 use strict;
 use GD;
@@ -65,6 +65,19 @@ sub my_options {
     }
 }
 
+sub demo_feature {
+    my $self = shift;
+    my $ex_image = 
+	'http://www.catch-fly.com/sites/awhittington/_files/Image/Drosophila-melanogaster.jpg';
+    return Bio::Graphics::Feature->new(-start=>1,
+				       -end=>500,
+				       -name=>$ex_image,
+				       -attributes => {
+					   image=>$ex_image,
+				       },
+	);
+}
+
 sub new {
   my $self  = shift->SUPER::new(@_);
   $self->{image} = $self->get_image();
